@@ -13,7 +13,7 @@
 #include "charsets.h"
 #include "glyph-cache.h"
 
-#define MISSING_GLYPH 4
+#define MISSING_GLYPH (NUM_UNDERLINE_STYLES + 2)
 #define MAX_NUM_EXTRA_GLYPHS_PUA 4u
 
 typedef void (*send_sprite_to_gpu_func)(FONTS_DATA_HANDLE fg, unsigned int, unsigned int, unsigned int, pixel*);
@@ -50,8 +50,8 @@ typedef struct {
     size_t font_idx;
 } SymbolMap;
 
-static SymbolMap *symbol_maps = NULL;
-static size_t num_symbol_maps = 0;
+static SymbolMap *symbol_maps = NULL, *narrow_symbols = NULL;
+static size_t num_symbol_maps = 0, num_narrow_symbols = 0;
 
 typedef enum { SPACER_STRATEGY_UNKNOWN, SPACERS_BEFORE, SPACERS_AFTER, SPACERS_IOSEVKA } SpacerStrategy;
 
@@ -422,7 +422,7 @@ load_fallback_font(FontGroup *fg, CPUCell *cell, bool bold, bool italic, bool em
     if (fg->fallback_fonts_count > 100) { log_error("Too many fallback fonts"); return MISSING_FONT; }
     ssize_t f;
 
-    if (bold) f = fg->italic_font_idx > 0 ? fg->bi_font_idx : fg->bold_font_idx;
+    if (bold) f = italic ? fg->bi_font_idx : fg->bold_font_idx;
     else f = italic ? fg->italic_font_idx : fg->medium_font_idx;
     if (f < 0) f = fg->medium_font_idx;
 
@@ -492,8 +492,8 @@ in_symbol_maps(FontGroup *fg, char_type ch) {
 // - BOX_FONT
 // - an index in the fonts list
 static ssize_t
-font_for_cell(FontGroup *fg, CPUCell *cpu_cell, GPUCell *gpu_cell, bool *is_fallback_font, bool *is_emoji_presentation) {
-    *is_fallback_font = false;
+font_for_cell(FontGroup *fg, CPUCell *cpu_cell, GPUCell *gpu_cell, bool *is_main_font, bool *is_emoji_presentation) {
+    *is_main_font = false;
     *is_emoji_presentation = false;
 START_ALLOW_CASE_RANGE
     ssize_t ans;
@@ -510,6 +510,7 @@ START_ALLOW_CASE_RANGE
         case 0x1fba0 ... 0x1fbae:
             return BOX_FONT;
         default:
+            *is_emoji_presentation = has_emoji_presentation(cpu_cell, gpu_cell);
             ans = in_symbol_maps(fg, cpu_cell->ch);
             if (ans > -1) return ans;
             switch(gpu_cell->attrs.bold | (gpu_cell->attrs.italic << 1)) {
@@ -523,9 +524,7 @@ START_ALLOW_CASE_RANGE
                     ans = fg->bi_font_idx; break;
             }
             if (ans < 0) ans = fg->medium_font_idx;
-            *is_emoji_presentation = has_emoji_presentation(cpu_cell, gpu_cell);
-            if (!*is_emoji_presentation && has_cell_text(fg->fonts + ans, cpu_cell)) return ans;
-            *is_fallback_font = true;
+            if (!*is_emoji_presentation && has_cell_text(fg->fonts + ans, cpu_cell)) { *is_main_font = true; return ans; }
             return fallback_font(fg, cpu_cell, gpu_cell);
     }
 END_ALLOW_CASE_RANGE
@@ -852,6 +851,19 @@ detect_spacer_strategy(hb_font_t *hbf, Font *font) {
             }
         }
     }
+
+    // If spacer_strategy is still default, check ### glyph to to confirm strategy
+    // https://github.com/kovidgoyal/kitty/issues/4721
+    if (font->spacer_strategy == SPACERS_BEFORE) {
+        cpu_cells[0].ch = '#'; cpu_cells[1].ch = '#'; cpu_cells[2].ch = '#';
+        shape(cpu_cells, gpu_cells, arraysz(cpu_cells), hbf, font, false);
+        if (G(num_glyphs) > 1) {
+            glyph_index glyph_id = G(info)[G(num_glyphs) - 1].codepoint;
+            bool is_special = is_special_glyph(glyph_id, font, &G(current_cell_data));
+            bool is_empty = is_special && is_empty_glyph(glyph_id, font);
+            if (is_empty) font->spacer_strategy = SPACERS_AFTER;
+        }
+    }
 }
 
 static LigatureType
@@ -963,7 +975,7 @@ group_normal(Font *font, hb_font_t *hbf) {
      *
      * We rely on the cluster numbers from harfbuzz to tell us how many unicode codepoints a glyph corresponds to.
      * Then we check if the glyph is a ligature glyph (is_special_glyph) and if it is an empty glyph.
-     * We detect if the font uses EMPTY glyphs before or after ligature glyphs (1. or 3. above) by checking what it does for ===.
+     * We detect if the font uses EMPTY glyphs before or after ligature glyphs (1. or 3. above) by checking what it does for === and ###.
      * Finally we look at the glyph name. These five datapoints give us enough information to satisfy the constraint above,
      * for a wide variety of fonts.
      */
@@ -1220,6 +1232,17 @@ is_non_emoji_dingbat(char_type ch) {
     return false;
 }
 
+static unsigned int
+cell_cap_for_codepoint(const char_type cp) {
+    unsigned int ans = UINT_MAX;
+    for (size_t i = 0; i < num_narrow_symbols; i++) {
+        SymbolMap *sm = narrow_symbols + i;
+        if (sm->left <= cp && cp <= sm->right) ans = sm->font_idx;
+    }
+    return ans;
+}
+
+
 void
 render_line(FONTS_DATA_HANDLE fg_, Line *line, index_type lnum, Cursor *cursor, DisableLigature disable_ligature_strategy) {
 #define RENDER if (run_font_idx != NO_FONT && i > first_cell_in_run) { \
@@ -1237,12 +1260,12 @@ render_line(FONTS_DATA_HANDLE fg_, Line *line, index_type lnum, Cursor *cursor,
         if (prev_width == 2) { prev_width = 0; continue; }
         CPUCell *cpu_cell = line->cpu_cells + i;
         GPUCell *gpu_cell = line->gpu_cells + i;
-        bool is_fallback_font, is_emoji_presentation;
-        ssize_t cell_font_idx = font_for_cell(fg, cpu_cell, gpu_cell, &is_fallback_font, &is_emoji_presentation);
+        bool is_main_font, is_emoji_presentation;
+        ssize_t cell_font_idx = font_for_cell(fg, cpu_cell, gpu_cell, &is_main_font, &is_emoji_presentation);
 
         if (
                 cell_font_idx != MISSING_FONT &&
-                ((is_fallback_font && !is_emoji_presentation && is_symbol(cpu_cell->ch)) || (cell_font_idx != BOX_FONT && (is_private_use(cpu_cell->ch))) || is_non_emoji_dingbat(cpu_cell->ch))
+                ((!is_main_font && !is_emoji_presentation && is_symbol(cpu_cell->ch)) || (cell_font_idx != BOX_FONT && (is_private_use(cpu_cell->ch))) || is_non_emoji_dingbat(cpu_cell->ch))
         ) {
             unsigned int desired_cells = 1;
             if (cell_font_idx > 0) {
@@ -1252,6 +1275,7 @@ render_line(FONTS_DATA_HANDLE fg_, Line *line, index_type lnum, Cursor *cursor,
                 int width = get_glyph_width(font->face, glyph_id);
                 desired_cells = (unsigned int)ceilf((float)width / fg->cell_width);
             }
+            desired_cells = MIN(desired_cells, cell_cap_for_codepoint(cpu_cell->ch));
 
             unsigned int num_spaces = 0;
             while (
@@ -1304,6 +1328,7 @@ render_simple_text(FONTS_DATA_HANDLE fg_, const char *text) {
 static void
 clear_symbol_maps(void) {
     if (symbol_maps) { free(symbol_maps); symbol_maps = NULL; num_symbol_maps = 0; }
+    if (narrow_symbols) { free(narrow_symbols); narrow_symbols = NULL; num_narrow_symbols = 0; }
 }
 
 typedef struct {
@@ -1312,26 +1337,33 @@ typedef struct {
 
 DescriptorIndices descriptor_indices = {0};
 
-static PyObject*
-set_font_data(PyObject UNUSED *m, PyObject *args) {
-    PyObject *sm;
-    Py_CLEAR(box_drawing_function); Py_CLEAR(prerender_function); Py_CLEAR(descriptor_for_idx); Py_CLEAR(font_feature_settings);
-    if (!PyArg_ParseTuple(args, "OOOIIIIO!dO",
-                &box_drawing_function, &prerender_function, &descriptor_for_idx,
-                &descriptor_indices.bold, &descriptor_indices.italic, &descriptor_indices.bi, &descriptor_indices.num_symbol_fonts,
-                &PyTuple_Type, &sm, &OPT(font_size), &font_feature_settings)) return NULL;
-    Py_INCREF(box_drawing_function); Py_INCREF(prerender_function); Py_INCREF(descriptor_for_idx); Py_INCREF(font_feature_settings);
-    free_font_groups();
-    clear_symbol_maps();
-    num_symbol_maps = PyTuple_GET_SIZE(sm);
-    symbol_maps = calloc(num_symbol_maps, sizeof(SymbolMap));
-    if (symbol_maps == NULL) return PyErr_NoMemory();
-    for (size_t s = 0; s < num_symbol_maps; s++) {
+static bool
+set_symbol_maps(SymbolMap **maps, size_t *num, const PyObject *sm) {
+    *num = PyTuple_GET_SIZE(sm);
+    *maps = calloc(*num, sizeof(SymbolMap));
+    if (*maps == NULL) { PyErr_NoMemory(); return false; }
+    for (size_t s = 0; s < *num; s++) {
         unsigned int left, right, font_idx;
-        SymbolMap *x = symbol_maps + s;
+        SymbolMap *x = *maps + s;
         if (!PyArg_ParseTuple(PyTuple_GET_ITEM(sm, s), "III", &left, &right, &font_idx)) return NULL;
         x->left = left; x->right = right; x->font_idx = font_idx;
     }
+    return true;
+}
+
+static PyObject*
+set_font_data(PyObject UNUSED *m, PyObject *args) {
+    PyObject *sm, *ns;
+    Py_CLEAR(box_drawing_function); Py_CLEAR(prerender_function); Py_CLEAR(descriptor_for_idx); Py_CLEAR(font_feature_settings);
+    if (!PyArg_ParseTuple(args, "OOOIIIIO!dOO!",
+                &box_drawing_function, &prerender_function, &descriptor_for_idx,
+                &descriptor_indices.bold, &descriptor_indices.italic, &descriptor_indices.bi, &descriptor_indices.num_symbol_fonts,
+                &PyTuple_Type, &sm, &OPT(font_size), &font_feature_settings, &PyTuple_Type, &ns)) return NULL;
+    Py_INCREF(box_drawing_function); Py_INCREF(prerender_function); Py_INCREF(descriptor_for_idx); Py_INCREF(font_feature_settings);
+    free_font_groups();
+    clear_symbol_maps();
+    set_symbol_maps(&symbol_maps, &num_symbol_maps, sm);
+    set_symbol_maps(&narrow_symbols, &num_narrow_symbols, ns);
     Py_RETURN_NONE;
 }
 
@@ -1398,6 +1430,11 @@ initialize_font_group(FontGroup *fg) {
     }
 #undef I
     calc_cell_metrics(fg);
+    // rescale the symbol_map faces for the desired cell height, this is how fallback fonts are sized as well
+    for (size_t i = 0; i < descriptor_indices.num_symbol_fonts; i++) {
+        Font *font = fg->fonts + i + fg->first_symbol_font_idx;
+        set_size_for_face(font->face, fg->cell_height, true, (FONTS_DATA_HANDLE)fg);
+    }
 }
 
 
