@@ -143,11 +143,6 @@ static bool
 set_font_size(Face *self, FT_F26Dot6 char_width, FT_F26Dot6 char_height, FT_UInt xdpi, FT_UInt ydpi, unsigned int desired_height, unsigned int cell_height) {
     int error = FT_Set_Char_Size(self->face, 0, char_height, xdpi, ydpi);
     if (!error) {
-        unsigned int ch = calc_cell_height(self, false);
-        if (desired_height && ch != desired_height) {
-            FT_F26Dot6 h = (FT_F26Dot6)floor((double)char_height * (double)desired_height / (double) ch);
-            return set_font_size(self, 0, h, xdpi, ydpi, 0, cell_height);
-        }
         self->char_width = char_width; self->char_height = char_height; self->xdpi = xdpi; self->ydpi = ydpi;
         if (self->harfbuzz_font != NULL) hb_ft_font_changed(self->harfbuzz_font);
     } else {
@@ -215,6 +210,14 @@ init_ft_face(Face *self, PyObject *path, int hinting, int hintstyle, FONTS_DATA_
     return true;
 }
 
+static void*
+set_load_error(const char *path, int error) {
+    char buf[2048];
+    snprintf(buf, sizeof(buf), "Failed to load face from path: %s with error:", path);
+    set_freetype_error(buf, error);
+    return NULL;
+}
+
 PyObject*
 face_from_descriptor(PyObject *descriptor, FONTS_DATA_HANDLE fg) {
 #define D(key, conv, missing_ok) { \
@@ -235,7 +238,7 @@ face_from_descriptor(PyObject *descriptor, FONTS_DATA_HANDLE fg) {
     Face *self = (Face *)Face_Type.tp_alloc(&Face_Type, 0);
     if (self != NULL) {
         int error = FT_New_Face(library, path, index, &(self->face));
-        if(error) { set_freetype_error("Failed to load face, with error:", error); Py_CLEAR(self); return NULL; }
+        if(error) { Py_CLEAR(self); return set_load_error(path, error); }
         if (!init_ft_face(self, PyDict_GetItemString(descriptor, "path"), hinting, hint_style, fg)) { Py_CLEAR(self); return NULL; }
     }
     return (PyObject*)self;
@@ -246,7 +249,7 @@ native_face_from_path(const char *path, int index) {
     int error;
     FT_Face ans;
     error = FT_New_Face(library, path, index, &ans);
-    if (error) { set_freetype_error("Failed to load face, with error:", error); return NULL; }
+    if (error) return set_load_error(path, error);
     return ans;
 }
 
@@ -256,7 +259,7 @@ face_from_path(const char *path, int index, FONTS_DATA_HANDLE fg) {
     if (ans == NULL) return NULL;
     int error;
     error = FT_New_Face(library, path, index, &ans->face);
-    if (error) { set_freetype_error("Failed to load face, with error:", error); ans->face = NULL; return NULL; }
+    if (error) { ans->face = NULL; return set_load_error(path, error); }
     if (!init_ft_face(ans, Py_None, true, 3, fg)) { Py_CLEAR(ans); return NULL; }
     return (PyObject*)ans;
 }
@@ -522,19 +525,20 @@ detect_right_edge(ProcessedBitmap *ans) {
 static bool
 render_color_bitmap(Face *self, int glyph_id, ProcessedBitmap *ans, unsigned int cell_width, unsigned int cell_height, unsigned int num_cells, unsigned int baseline UNUSED) {
     unsigned short best = 0, diff = USHRT_MAX;
-    const short limit = self->face->num_fixed_sizes;
-    for (short i = 0; i < limit; i++) {
-        unsigned short w = self->face->available_sizes[i].width;
-        unsigned short d = w > (unsigned short)cell_width ? w - (unsigned short)cell_width : (unsigned short)cell_width - w;
-        if (d < diff) {
-            diff = d;
-            best = i;
+    if (self->face->num_fixed_sizes > 0) {
+        const short limit = self->face->num_fixed_sizes;
+        for (short i = 0; i < limit; i++) {
+            unsigned short w = self->face->available_sizes[i].width;
+            unsigned short d = w > (unsigned short)cell_width ? w - (unsigned short)cell_width : (unsigned short)cell_width - w;
+            if (d < diff) {
+                diff = d;
+                best = i;
+            }
         }
-    }
-    FT_Error error = FT_Select_Size(self->face, best);
-    if (error) { set_freetype_error("Failed to set char size for non-scalable font, with error:", error); return false; }
-    if (!load_glyph(self, glyph_id, FT_LOAD_COLOR)) return false;
-    FT_Set_Char_Size(self->face, 0, self->char_height, self->xdpi, self->ydpi);
+        FT_Error error = FT_Select_Size(self->face, best);
+        if (error) { set_freetype_error("Failed to set char size for non-scalable font, with error:", error); return false; }
+    } else FT_Set_Char_Size(self->face, 0, self->char_height, self->xdpi, self->ydpi);
+    if (!load_glyph(self, glyph_id, FT_LOAD_COLOR | FT_LOAD_RENDER)) return false;
     FT_Bitmap *bitmap = &self->face->glyph->bitmap;
     if (bitmap->pixel_mode != FT_PIXEL_MODE_BGRA) return false;
     ans->buf = bitmap->buffer;
