@@ -17,7 +17,7 @@ from .conf.utils import BadLine
 from .config import cached_values_for
 from .constants import (
     appname, beam_cursor_data_file, config_dir, glfw_path, is_macos,
-    is_wayland, kitty_exe, logo_png_file, running_in_kitty
+    is_wayland, kitty_exe, logo_png_file, running_in_kitty, website_url
 )
 from .fast_data_types import (
     GLFW_IBEAM_CURSOR, GLFW_MOD_ALT, GLFW_MOD_SHIFT, create_os_window,
@@ -29,12 +29,11 @@ from .fonts.render import set_font_family
 from .options.types import Options
 from .options.utils import DELETE_ENV_VAR
 from .os_window_size import initial_window_size_func
-from .session import get_os_window_sizing_data
-from .shell_integration import setup_shell_integration
+from .session import create_sessions, get_os_window_sizing_data
 from .types import SingleKey
 from .utils import (
-    detach, expandvars, log_error, single_instance,
-    startup_notification_handler, unix_socket_paths
+    cleanup_ssh_control_masters, detach, expandvars, log_error,
+    single_instance, startup_notification_handler, unix_socket_paths
 )
 from .window import load_shader_programs
 
@@ -60,7 +59,7 @@ def talk_to_instance(args: CLIOptions) -> None:
     stdin = ''
     if args.session == '-':
         stdin = sys.stdin.read()
-    data = {'cmd': 'new_instance', 'args': tuple(sys.argv),
+    data = {'cmd': 'new_instance', 'args': tuple(sys.argv), 'cmdline_args_for_open': getattr(sys, 'cmdline_args_for_open', []),
             'startup_id': os.environ.get('DESKTOP_STARTUP_ID'),
             'cwd': os.getcwd(), 'stdin': stdin}
     notify_socket = None
@@ -151,29 +150,37 @@ def _run_app(opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = ())
             func_map[parts].append(k)
 
         for ac in ('new_os_window', 'close_os_window', 'close_tab', 'edit_config_file', 'previous_tab',
-                   'next_tab', 'new_tab', 'new_window', 'close_window', 'toggle_macos_secure_keyboard_entry'):
+                   'next_tab', 'new_tab', 'new_window', 'close_window', 'toggle_macos_secure_keyboard_entry', 'toggle_fullscreen'):
             val = get_macos_shortcut_for(func_map, ac)
             if val is not None:
                 global_shortcuts[ac] = val
         val = get_macos_shortcut_for(func_map, 'clear_terminal reset active', lookup_name='reset_terminal')
         if val is not None:
             global_shortcuts['reset_terminal'] = val
+        val = get_macos_shortcut_for(func_map, 'clear_terminal to_cursor active', lookup_name='clear_terminal_and_scrollback')
+        if val is not None:
+            global_shortcuts['clear_terminal_and_scrollback'] = val
         val = get_macos_shortcut_for(func_map, 'load_config_file', lookup_name='reload_config')
         if val is not None:
             global_shortcuts['reload_config'] = val
+        val = get_macos_shortcut_for(func_map, f'open_url {website_url()}', lookup_name='open_kitty_website')
+        if val is not None:
+            global_shortcuts['open_kitty_website'] = val
     if is_macos and opts.macos_custom_beam_cursor:
         set_custom_ibeam_cursor()
     if not is_wayland() and not is_macos:  # no window icons on wayland
         set_x11_window_icon()
     with cached_values_for(run_app.cached_values_name) as cached_values:
+        startup_sessions = tuple(create_sessions(opts, args, default_session=opts.startup_session))
+        wincls = (startup_sessions[0].os_window_class if startup_sessions else '') or args.cls or appname
         with startup_notification_handler(extra_callback=run_app.first_window_callback) as pre_show_callback:
             window_id = create_os_window(
-                    run_app.initial_window_size_func(get_os_window_sizing_data(opts), cached_values),
+                    run_app.initial_window_size_func(get_os_window_sizing_data(opts, startup_sessions[0] if startup_sessions else None), cached_values),
                     pre_show_callback,
                     args.title or appname, args.name or args.cls or appname,
-                    args.cls or appname, load_all_shaders, disallow_override_title=bool(args.title))
+                    wincls, load_all_shaders, disallow_override_title=bool(args.title))
         boss = Boss(opts, args, cached_values, global_shortcuts)
-        boss.start(window_id)
+        boss.start(window_id, startup_sessions)
         if bad_lines:
             boss.show_bad_config_lines(bad_lines)
         try:
@@ -319,7 +326,6 @@ def setup_environment(opts: Options, cli_opts: CLIOptions) -> None:
         os.environ['KITTY_LISTEN_ON'] = cli_opts.listen_on
     env = opts.env.copy()
     ensure_kitty_in_path()
-    setup_shell_integration(opts, env)
     kitty_path = shutil.which('kitty')
     if kitty_path:
         child_path = env.get('PATH')
@@ -337,11 +343,13 @@ def set_locale() -> None:
         locale.setlocale(locale.LC_ALL, '')
     except Exception:
         log_error('Failed to set locale with LANG:', os.environ.get('LANG'))
-        os.environ.pop('LANG', None)
-        try:
-            locale.setlocale(locale.LC_ALL, '')
-        except Exception:
-            log_error('Failed to set locale with no LANG')
+        old_lang = os.environ.pop('LANG', None)
+        if old_lang is not None:
+            try:
+                locale.setlocale(locale.LC_ALL, '')
+            except Exception:
+                log_error('Failed to set locale with no LANG')
+            os.environ['LANG'] = old_lang
 
 
 def _main() -> None:
@@ -364,8 +372,21 @@ def _main() -> None:
         cwd_ok = False
     if not cwd_ok:
         os.chdir(os.path.expanduser('~'))
-    cli_opts, rest = parse_args(args=args, result_class=CLIOptions)
-    cli_opts.args = rest
+    if getattr(sys, 'cmdline_args_for_open', False):
+        usage: Optional[str] = 'file_or_url ...'
+        appname: Optional[str] = 'kitty +open'
+        msg: Optional[str] = (
+            'Run kitty and open the specified files or URLs in it, using launch-actions.conf. For details'
+            ' see https://sw.kovidgoyal.net/kitty/open_actions/#scripting-the-opening-of-files-with-kitty-on-macos'
+            '\n\nAll the normal kitty options can be used.')
+    else:
+        usage = msg = appname = None
+    cli_opts, rest = parse_args(args=args, result_class=CLIOptions, usage=usage, message=msg, appname=appname)
+    if getattr(sys, 'cmdline_args_for_open', False):
+        setattr(sys, 'cmdline_args_for_open', rest)
+        cli_opts.args = []
+    else:
+        cli_opts.args = rest
     if cli_opts.detach:
         if cli_opts.session == '-':
             from .session import PreReadSession
@@ -394,6 +415,7 @@ def _main() -> None:
             run_app(opts, cli_opts, bad_lines)
     finally:
         glfw_terminate()
+        cleanup_ssh_control_masters()
 
 
 def main() -> None:
