@@ -7,6 +7,7 @@
 #pragma once
 
 #include "../state.h"
+#include "../colors.h"
 
 static inline float
 PyFloat_AsFloat(PyObject *o) {
@@ -15,10 +16,15 @@ PyFloat_AsFloat(PyObject *o) {
 
 static inline color_type
 color_as_int(PyObject *color) {
-    if (!PyTuple_Check(color)) { PyErr_SetString(PyExc_TypeError, "Not a color tuple"); return 0; }
-#define I(n, s) ((PyLong_AsUnsignedLong(PyTuple_GET_ITEM(color, n)) & 0xff) << s)
-    return (I(0, 16) | I(1, 8) | I(2, 0)) & 0xffffff;
-#undef I
+    if (!PyObject_TypeCheck(color, &Color_Type)) { PyErr_SetString(PyExc_TypeError, "Not a Color object"); return 0; }
+    Color *c = (Color*)color;
+    return c->color.val & 0xffffff;
+}
+
+static inline color_type
+color_or_none_as_int(PyObject *color) {
+    if (color == Py_None) return 0;
+    return color_as_int(color);
 }
 
 static inline color_type
@@ -66,22 +72,48 @@ bglayout(PyObject *layout_name) {
         case 't': return TILING;
         case 'm': return MIRRORED;
         case 's': return SCALED;
+        case 'c': return CLAMPED;
         default: break;
     }
     return TILING;
 }
 
-static void
-background_image(PyObject *src, Options *opts) {
-    if (opts->background_image) free(opts->background_image);
-    opts->background_image = NULL;
-    if (src == Py_None || !PyUnicode_Check(src)) return;
-    Py_ssize_t sz;
-    const char *s = PyUnicode_AsUTF8AndSize(src, &sz);
-    opts->background_image = calloc(sz + 1, 1);
-    if (opts->background_image) memcpy(opts->background_image, s, sz);
+static ImageAnchorPosition
+bganchor(PyObject *anchor_name) {
+    const char *name = PyUnicode_AsUTF8(anchor_name);
+    ImageAnchorPosition anchor = {0.5f, 0.5f, 0.5f, 0.5f};
+    if (strstr(name, "top") != NULL) {
+        anchor.canvas_y = 0.f; anchor.image_y = 0.f;
+    } else if (strstr(name, "bottom") != NULL) {
+        anchor.canvas_y = 1.f; anchor.image_y = 1.f;
+    }
+    if (strstr(name, "left") != NULL) {
+        anchor.canvas_x = 0.f; anchor.image_x = 0.f;
+    } else if (strstr(name, "right") != NULL) {
+        anchor.canvas_x = 1.f; anchor.image_x = 1.f;
+    }
+    return anchor;
 }
 
+#define STR_SETTER(name) { \
+    free(opts->name); opts->name = NULL; \
+    if (src == Py_None || !PyUnicode_Check(src)) return; \
+    Py_ssize_t sz; \
+    const char *s = PyUnicode_AsUTF8AndSize(src, &sz); \
+    opts->name = calloc(sz + 1, 1); \
+    if (opts->name) memcpy(opts->name, s, sz); \
+}
+
+static void
+background_image(PyObject *src, Options *opts) { STR_SETTER(background_image); }
+
+static void
+bell_path(PyObject *src, Options *opts) { STR_SETTER(bell_path); }
+
+static void
+window_logo_path(PyObject *src, Options *opts) { STR_SETTER(default_window_logo); }
+
+#undef STR_SETTER
 
 static MouseShape
 pointer_shape(PyObject *shape_name) {
@@ -95,6 +127,13 @@ pointer_shape(PyObject *shape_name) {
     return BEAM;
 }
 
+static int
+macos_colorspace(PyObject *csname) {
+    if (PyUnicode_CompareWithASCIIString(csname, "srgb")) return 1;
+    if (PyUnicode_CompareWithASCIIString(csname, "displayp3")) return 2;
+    return 0;
+}
+
 static inline void
 free_url_prefixes(void) {
     OPT(url_prefixes).num = 0;
@@ -148,6 +187,12 @@ select_by_word_characters(PyObject *chars, Options *opts) {
     opts->select_by_word_characters = list_of_chars(chars);
 }
 
+static void
+select_by_word_characters_forward(PyObject *chars, Options *opts) {
+    free(opts->select_by_word_characters_forward);
+    opts->select_by_word_characters_forward = list_of_chars(chars);
+}
+
 static void
 tab_bar_style(PyObject *val, Options *opts) {
     opts->tab_bar_hidden = PyUnicode_CompareWithASCIIString(val, "hidden") == 0 ? true: false;
