@@ -1001,7 +1001,7 @@ dispatch_csi(Screen *screen, PyObject DUMP_UNUSED *dump_callback) {
         case 'm':
             if (start_modifier == '>' && !end_modifier) {
                 REPORT_ERROR(
-                    "The application is trying to use XTerm's modifyOtherKeys."
+                    "The application is trying to use xterm's modifyOtherKeys."
                     " This is superseded by the kitty keyboard protocol: https://sw.kovidgoyal.net/kitty/keyboard-protocol/"
                     " the application should be updated to use that"
                 );
@@ -1086,6 +1086,7 @@ dispatch_dcs(Screen *screen, PyObject DUMP_UNUSED *dump_callback) {
                 } else IF_SIMPLE_PREFIX("echo|", handle_remote_echo)
                 } else IF_SIMPLE_PREFIX("ssh|", handle_remote_ssh)
                 } else IF_SIMPLE_PREFIX("ask|", handle_remote_askpass)
+                } else IF_SIMPLE_PREFIX("clone|", handle_remote_clone)
 #undef IF_SIMPLE_PREFIX
                 } else {
                     REPORT_ERROR("Unrecognized DCS @ code: 0x%x", screen->parser_buf[1]);
