@@ -602,7 +602,7 @@ send_pending_click_to_window_id(id_type timer_id UNUSED, void *data) {
 }
 
 bool
-update_ime_position_for_window(id_type window_id, bool force) {
+update_ime_position_for_window(id_type window_id, bool force, int update_focus) {
     for (size_t o = 0; o < global_state.num_os_windows; o++) {
         OSWindow *osw = global_state.os_windows + o;
         for (size_t t = 0; t < osw->num_tabs; t++) {
@@ -610,10 +610,14 @@ update_ime_position_for_window(id_type window_id, bool force) {
             for (size_t w = 0; w < qtab->num_windows; w++) {
                 Window *window = qtab->windows + w;
                 if (window->id == window_id) {
-                    if (window->render_data.screen && (force || osw->is_focused)) {
+                    // The screen may not be ready after the new window is created and focused, and still needs to enable IME.
+                    if ((window->render_data.screen && (force || osw->is_focused)) || update_focus > 0) {
                         OSWindow *orig = global_state.callback_os_window;
                         global_state.callback_os_window = osw;
-                        update_ime_position(window, window->render_data.screen);
+                        if (update_focus) update_ime_focus(osw, update_focus > 0);
+                        if (update_focus >= 0 && window->render_data.screen) {
+                            update_ime_position(window, window->render_data.screen);
+                        }
                         global_state.callback_os_window = orig;
                         return true;
                     }
@@ -652,8 +656,9 @@ update_ime_position_for_window(id_type window_id, bool force) {
 PYWRAP1(update_ime_position_for_window) {
     id_type window_id;
     int force = 0;
-    PA("K|p", &window_id, &force);
-    if (update_ime_position_for_window(window_id, force)) Py_RETURN_TRUE;
+    int update_focus = 0;
+    PA("K|pi", &window_id, &force, &update_focus);
+    if (update_ime_position_for_window(window_id, force, update_focus)) Py_RETURN_TRUE;
     Py_RETURN_FALSE;
 }
 
@@ -703,7 +708,11 @@ PYWRAP1(set_options) {
     Py_RETURN_NONE;
 }
 
-BOOL_SET(in_sequence_mode)
+PYWRAP1(set_in_sequence_mode) {
+    global_state.in_sequence_mode = PyObject_IsTrue(args);
+    set_ignore_os_keyboard_processing(global_state.in_sequence_mode);
+    Py_RETURN_NONE;
+}
 
 static void
 init_screen_render_data(OSWindow *osw, const WindowGeometry *g, ScreenRenderData *d) {
@@ -918,12 +927,15 @@ PYWRAP1(sync_os_window_title) {
 
 PYWRAP1(set_os_window_title) {
     id_type os_window_id;
-    const char *title;
-    PA("Ks", &os_window_id, &title);
+    PyObject *title;
+    PA("KU", &os_window_id, &title);
     WITH_OS_WINDOW(os_window_id)
-        if (strlen(title)) {
+        if (PyUnicode_GetLength(title)) {
             os_window->title_is_overriden = true;
-            set_os_window_title(os_window, title);
+            Py_XDECREF(os_window->window_title);
+            os_window->window_title = title;
+            Py_INCREF(title);
+            set_os_window_title(os_window, PyUnicode_AsUTF8(title));
         } else {
             os_window->title_is_overriden = false;
             if (os_window->window_title) set_os_window_title(os_window, PyUnicode_AsUTF8(os_window->window_title));
@@ -933,6 +945,15 @@ PYWRAP1(set_os_window_title) {
     Py_RETURN_NONE;
 }
 
+PYWRAP1(get_os_window_title) {
+    id_type os_window_id;
+    PA("K", &os_window_id);
+    WITH_OS_WINDOW(os_window_id)
+        if (os_window->window_title) return Py_BuildValue("O", os_window->window_title);
+    END_WITH_OS_WINDOW
+    Py_RETURN_NONE;
+}
+
 
 
 PYWRAP1(pt_to_px) {
@@ -1271,6 +1292,7 @@ static PyMethodDef module_methods[] = {
     MW(background_opacity_of, METH_O),
     MW(update_window_visibility, METH_VARARGS),
     MW(sync_os_window_title, METH_VARARGS),
+    MW(get_os_window_title, METH_VARARGS),
     MW(set_os_window_title, METH_VARARGS),
     MW(global_font_size, METH_VARARGS),
     MW(set_background_image, METH_VARARGS),
