@@ -33,6 +33,7 @@ typedef struct {
     unsigned int scrollback_pager_history_size;
     bool scrollback_fill_enlarged_window;
     char_type *select_by_word_characters;
+    char_type *select_by_word_characters_forward;
     color_type url_color, background, foreground, active_border_color, inactive_border_color, bell_border_color, tab_bar_background, tab_bar_margin_color;
     color_type mark1_foreground, mark1_background, mark2_foreground, mark2_background, mark3_foreground, mark3_background;
     monotonic_t repaint_delay, input_delay;
@@ -82,6 +83,7 @@ typedef struct {
         double outer, inner;
     } tab_bar_margin_height;
     long macos_menubar_title_max_length;
+    int macos_colorspace;
 } Options;
 
 typedef struct WindowLogoRenderData {
@@ -233,7 +235,7 @@ typedef struct {
     bool check_for_active_animated_images;
     struct { double x, y; } default_dpi;
     id_type active_drag_in_window, tracked_drag_in_window;
-    int active_drag_button;
+    int active_drag_button, tracked_drag_button;
     CloseRequest quit_request;
     bool redirect_mouse_handling;
     WindowLogoTable *all_window_logos;
@@ -309,6 +311,7 @@ typedef enum {
     RELOAD_CONFIG,
     TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY,
     TOGGLE_FULLSCREEN,
+    OPEN_KITTY_WEBSITE,
 
     NUM_COCOA_PENDING_ACTIONS
 } CocoaPendingAction;
