@@ -45,6 +45,9 @@ class Callbacks:
     def set_color_table_color(self, code, data) -> None:
         self.ctbuf += ''
 
+    def color_profile_popped(self, x) -> None:
+        pass
+
     def request_capabilities(self, q) -> None:
         from kitty.terminfo import get_capabilities
         for c in get_capabilities(q, None):
@@ -71,6 +74,8 @@ class Callbacks:
         self.open_urls = []
         self.cc_buf = []
         self.bell_count = 0
+        self.clone_cmds = []
+        self.current_clone_data = ''
 
     def on_bell(self) -> None:
         self.bell_count += 1
@@ -94,6 +99,19 @@ class Callbacks:
         text = process_remote_print(msg)
         print(text, file=sys.__stderr__)
 
+    def handle_remote_clone(self, msg):
+        if not msg:
+            if self.current_clone_data:
+                cdata, self.current_clone_data = self.current_clone_data, ''
+                from kitty.launch import CloneCmd
+                self.clone_cmds.append(CloneCmd(cdata))
+            self.current_clone_data = ''
+            return
+        num, rest = msg.split(':', 1)
+        if num == '0' or len(self.current_clone_data) > 1024 * 1024:
+            self.current_clone_data = ''
+        self.current_clone_data += rest
+
     def handle_remote_ssh(self, msg):
         from kittens.ssh.main import get_ssh_data
         if self.pty:
@@ -159,7 +177,7 @@ class BaseTest(TestCase):
         s = Screen(c, lines, cols, scrollback, cell_width, cell_height, 0, c)
         return s
 
-    def create_pty(self, argv, cols=80, lines=25, scrollback=100, cell_width=10, cell_height=20, options=None, cwd=None, env=None):
+    def create_pty(self, argv, cols=80, lines=100, scrollback=100, cell_width=10, cell_height=20, options=None, cwd=None, env=None):
         self.set_options(options)
         return PTY(argv, lines, cols, scrollback, cell_width, cell_height, cwd, env)
 
