@@ -5,6 +5,7 @@ from kitty.fast_data_types import (
     DECAWM, DECCOLM, DECOM, IRM, Cursor, parse_bytes
 )
 from kitty.marks import marker_from_function, marker_from_regex
+from kitty.window import pagerhist
 
 from . import BaseTest
 
@@ -292,6 +293,18 @@ class TestScreen(BaseTest):
         self.ae(str(s.linebuf), '55555\n66666\n77777\n88888\n99999')
         s.resize(5, 2)
         self.ae(str(s.linebuf), '88\n88\n99\n99\n9')
+        s = self.create_screen()
+        s.draw('a' * s.columns)
+        s.linefeed(), s.carriage_return()
+        s.draw('bb')
+        s.resize(s.lines, s.columns - 2)
+        self.ae(str(s.linebuf), 'aaa\naa\nbb\n\n')
+        s.cursor.y = s.cursor.x = 0
+        s.draw('x' * len(str(s.line(0))))
+        s.linefeed(), s.carriage_return()
+        s.draw('x' * len(str(s.line(1))))
+        s.resize(s.lines, s.columns + 4)
+        self.ae(str(s.linebuf), 'xxx\nxx\nbb\n\n')
 
     def test_cursor_after_resize(self):
 
@@ -530,6 +543,28 @@ class TestScreen(BaseTest):
         s.scroll(2, True)
         self.ae(s.text_for_selection(), expected)
         s.reset()
+        s.draw('ab   cd')
+        s.start_selection(0, 0)
+        s.update_selection(1, 3)
+        self.ae(s.text_for_selection(), ('ab   ', 'cd'))
+        self.ae(s.text_for_selection(False, True), ('ab', 'cd'))
+        s.reset()
+        s.draw('ab        cd')
+        s.start_selection(0, 0)
+        s.update_selection(3, 4)
+        self.ae(s.text_for_selection(), ('ab   ', '     ', 'cd'))
+        self.ae(s.text_for_selection(False, True), ('ab', '\n', 'cd'))
+        s.reset()
+        s.draw('a')
+        s.select_graphic_rendition(32)
+        s.draw('b')
+        s.select_graphic_rendition(39)
+        s.draw('c  xy')
+        s.start_selection(0, 0)
+        s.update_selection(1, 3)
+        self.ae(s.text_for_selection(), ('abc  ', 'xy'))
+        self.ae(s.text_for_selection(True), ('a\x1b[32mb\x1b[39mc  ', 'xy', '\x1b[m'))
+        self.ae(s.text_for_selection(True, True), ('a\x1b[32mb\x1b[39mc', 'xy', '\x1b[m'))
 
     def test_soft_hyphen(self):
         s = self.create_screen()
@@ -974,9 +1009,12 @@ class TestScreen(BaseTest):
         self.assertTrue(s.scroll_to_prompt())
         self.ae(str(s.visual_line(0)), '$ 1')
 
-        def lco():
+        def lco(as_ansi=False):
             a = []
-            s.cmd_output(0, a.append)
+            if s.cmd_output(0, a.append, as_ansi):
+                pht = pagerhist(s, as_ansi=as_ansi, upto_output_start=True)
+                if pht:
+                    a.insert(0, pht)
             return ''.join(a)
 
         def fco():
@@ -1003,6 +1041,7 @@ class TestScreen(BaseTest):
         mark_prompt(), s.draw('$ 1')
         self.ae(fco(), 'abcd\n12')
         self.ae(lco(), 'abcd\n12')
+        self.ae(lco(as_ansi=True), '\x1b[m\x1b]133;C\x1b\\abcd\n\x1b[m12')
 
         def draw_prompt(x):
             mark_prompt(), s.draw(f'$ {x}'), s.carriage_return(), s.index()
@@ -1053,3 +1092,9 @@ class TestScreen(BaseTest):
         self.ae(lvco(), '0\n1\n2')
         s.scroll_to_prompt(1)
         self.ae(lvco(), '0x\n1x')
+
+        # last command output from pager history
+        s = self.create_screen()
+        draw_prompt('p1')
+        draw_output(30)
+        self.ae(tuple(map(int, lco().split())), tuple(range(0, 30)))
